@@ -2,7 +2,7 @@
 
 pub mod covariance;
 pub mod ud;
-pub mod unscented;
+pub mod unscented_duplex;
 pub mod information;
 pub mod information_root;
 #[cfg(feature = "std")]
