@@ -267,7 +267,7 @@ pub fn systematic_resampler(l: &mut Likelihoods, rng: &mut dyn RngCore) -> Resul
 
 /// Normalised cumulative sum of likelihoods (Kahan algorithm), and find smallest likelihood.
 fn cumaltive_likelihood(l: &mut Likelihoods) -> Result<(f32, f32), &'static str> {
-    let mut lmin = f32::max_value();
+    let mut lmin = <f32 as Real>::max_value();
     let mut lcum = 0.;
     {
         let mut c = 0.;
