@@ -223,7 +223,7 @@ impl<N: Copy + RealField, D: Dim> UDState<N, D>
         }
 
         // Apply observations sequential as they are decorrelated
-        let mut rcondmin = N::max_value();
+        let mut rcondmin = N::max_value().unwrap();
         for o in 0..z_size {
             // Update UD and extract gain
             let mut S = self.udu.zero;
@@ -326,7 +326,7 @@ impl<N: Copy + RealField, D: Dim> UDState<N, D>
         let z_size = s.nrows();
 
         // Apply observations sequentially as they are decorrelated
-        let mut rcondmin = N::max_value();
+        let mut rcondmin = N::max_value().unwrap();
 
         for o in 0..z_size {
             // Check noise precondition
