@@ -9,6 +9,46 @@ use nalgebra::{allocator::Allocator, DefaultAllocator, Dim, OMatrix, OVector, Re
 
 use crate::noise::CorrelatedNoise;
 
+/// A state estimator.
+pub trait Estimator<N: RealField, D: Dim>
+    where
+        DefaultAllocator: Allocator<N, D>,
+{
+    /// The estimator's estimate of the system's state.
+    fn state(&self) -> Result<OVector<N, D>, &'static str>;
+}
+
+/// A functional state predictor.
+///
+/// Uses a functional state prediction model with additive noise.
+pub trait FunctionalPredictor<N: RealField, D: Dim>
+    where
+        DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>
+{
+    /// Uses the functional state prediction model with additive noise.
+    fn predict(
+        &mut self, f: impl Fn(&OVector<N, D>) -> OVector<N, D>,
+        noise: &CorrelatedNoise<N, D>)
+        -> Result<(), &'static str>;
+}
+
+/// A functional state observer.
+///
+/// Uses a functional state observation model with additive noise.
+pub trait FunctionalObserver<N: RealField, D: Dim, ZD: Dim>
+    where
+        DefaultAllocator:
+        Allocator<N, ZD, D> + Allocator<N, ZD, ZD> + Allocator<N, D> + Allocator<N, ZD>
+{
+    /// Uses the functional state observation with additive noise.
+    fn observe(
+        &mut self,
+        z: &OVector<N, ZD>,
+        h: impl Fn(&OVector<N, D>) -> OVector<N, ZD>,
+        noise: &CorrelatedNoise<N, ZD>)
+        -> Result<(), &'static str>;
+}
+
 /// Kalman state.
 ///
 /// Linear representation as a state vector and the state covariance (symmetric positive semi-definite) matrix.
@@ -38,15 +78,6 @@ where
     pub I: OMatrix<N, D, D>,
 }
 
-/// A state estimator.
-pub trait Estimator<N: RealField, D: Dim>
-    where
-        DefaultAllocator: Allocator<N, D>,
-{
-    /// The estimator's estimate of the system's state.
-    fn state(&self) -> Result<OVector<N, D>, &'static str>;
-}
-
 /// A Kalman estimator.
 ///
 /// The linear Kalman state representation x,X is used to represent the system.
@@ -63,12 +94,12 @@ where
 
 /// An extended linear predictor.
 ///
-/// Uses a non-linear state prediction with linear estimation model with additive noise.
+/// Uses a non-linear state prediction with linearised estimation model and additive noise.
 pub trait ExtendedLinearPredictor<N: RealField, D: Dim>
 where
     DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>
 {
-    /// Uses a non-linear state prediction with linear estimation model with additive noise.
+    /// Uses a non-linear state prediction with linear estimation model and additive noise.
     fn predict(
         &mut self,
         x_pred: &OVector<N, D>,
@@ -77,15 +108,15 @@ where
     ) -> Result<(), &'static str>;
 }
 
-/// A extended linear observer with correlated observation noise.
+/// An extended linear observer.
 ///
-/// Uses a non-linear state observation with linear estimation model with additive noise.
+/// Uses a non-linear state observation with linearised estimation model and additive noise.
 pub trait ExtendedLinearObserver<N: RealField, D: Dim, ZD: Dim>
 where
     DefaultAllocator:
         Allocator<N, ZD, D> + Allocator<N, ZD, ZD> + Allocator<N, ZD>
 {
-    /// Uses a non-linear state observation with linear estimation model with additive noise.
+    /// Uses a non-linear state observation with linear estimation model and additive noise.
     fn observe_innovation(
         &mut self,
         s: &OVector<N, ZD>,
