@@ -21,11 +21,12 @@ use bayes_estimate::estimators::information_root::InformationRootState;
 use bayes_estimate::estimators::sir;
 use bayes_estimate::estimators::ud::{CorrelatedFactorNoise, UDState};
 use bayes_estimate::models::{
-    Estimator, ExtendedLinearObserver, ExtendedLinearPredictor,
+    Estimator, ExtendedLinearObserver, ExtendedLinearPredictor, FunctionalPredictor, FunctionalObserver,
     InformationState, KalmanEstimator, KalmanState,
 };
 use bayes_estimate::noise::{CorrelatedNoise, CoupledNoise, UncorrelatedNoise};
 use num_traits::FromPrimitive;
+use bayes_estimate::estimators::unscented_duplex::UnscentedDuplexState;
 
 /// Define the estimator operations to be tested.
 pub trait FatEstimator<D: Dim, QD: Dim, ZD: Dim>: Estimator<f64, D> + KalmanEstimator<f64, D>
@@ -271,8 +272,7 @@ impl<D: DimAdd<U1>, QD: Dim, ZD: Dim> FatEstimator<D, QD, ZD> for FatUDState<f64
 pub struct FatUnscentedState<N: RealField, D: Dim>
     where DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>
 {
-    pub kalman: KalmanState<N, D>,
-    pub kappa: N,
+    pub unscented: UnscentedDuplexState<N, D>
 }
 
 impl<N: FromPrimitive + RealField, D: Dim> FatUnscentedState<N, D>
@@ -281,8 +281,10 @@ impl<N: FromPrimitive + RealField, D: Dim> FatUnscentedState<N, D>
 {
     pub fn new_zero(d: D) -> FatUnscentedState<N, D> {
         FatUnscentedState {
-            kalman: KalmanState::new_zero(d),
-            kappa: N::from_usize(3 - d.value()).unwrap(),
+            unscented: UnscentedDuplexState {
+                kalman: KalmanState::new_zero(d),
+                kappa: N::from_usize(3 - d.value()).unwrap()
+            }
         }
     }
 }
@@ -291,7 +293,7 @@ impl<N: RealField, D: Dim> Estimator<N, D> for FatUnscentedState<N, D>
     where DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>
 {
     fn state(&self) -> Result<OVector<N, D>, &'static str> {
-        return Ok(self.kalman.x.clone());
+        return Ok(self.unscented.kalman.x.clone());
     }
 }
 
@@ -299,14 +301,14 @@ impl<N: RealField, D: Dim> KalmanEstimator<N, D> for FatUnscentedState<N, D>
     where DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>
 {
     fn init(&mut self, state: &KalmanState<N, D>) -> Result<(), &'static str> {
-        self.kalman.x.copy_from(&state.x);
-        self.kalman.X.copy_from(&state.X);
+        self.unscented.kalman.x.copy_from(&state.x);
+        self.unscented.kalman.X.copy_from(&state.X);
 
         Ok(())
     }
 
     fn kalman_state(&self) -> Result<KalmanState<N, D>, &'static str> {
-        return Ok(self.kalman.clone());
+        return Ok(self.unscented.kalman.clone());
     }
 }
 
@@ -325,7 +327,7 @@ impl<D: Dim, QD: Dim, ZD: Dim> FatEstimator<D, QD, ZD> for FatUnscentedState<f64
         DefaultAllocator: Allocator<usize, D, D>,
 {
     fn trace_state(&self) {
-        println!("{:}", self.kalman.X);
+        println!("{:}", self.unscented.kalman.X);
     }
 
     fn predict_fn(
@@ -335,7 +337,7 @@ impl<D: Dim, QD: Dim, ZD: Dim> FatEstimator<D, QD, ZD> for FatUnscentedState<f64
         _fx: &OMatrix<f64, D, D>,
         noise: &CoupledNoise<f64, D, QD>)
     {
-        self.kalman.predict_unscented(f, &CorrelatedNoise::from_coupled::<QD>(noise), self.kappa).unwrap();
+        self.unscented.predict(f, &CorrelatedNoise::from_coupled::<QD>(noise)).unwrap();
     }
 
     fn observe(
@@ -346,8 +348,12 @@ impl<D: Dim, QD: Dim, ZD: Dim> FatEstimator<D, QD, ZD> for FatUnscentedState<f64
         _hx: &OMatrix<f64, ZD, D>,
         noise: &CorrelatedNoise<f64, ZD>,
     ) -> Result<(), &'static str> {
-        let s = &(z - h(&self.state().unwrap()));
-        self.kalman.observe_unscented(h, h_normalize, noise, s, self.kappa)
+        let hn = |x: &OVector<f64, D>| -> OVector<f64, ZD> {
+            let mut zp = h(x);
+            h_normalize(&mut zp, z);
+            zp
+        };
+        self.unscented.observe(&z, hn, noise)
     }
 }
 
